#!/usr/bin/perl -w

our $VERSION='1.20130107';

=head1 NAME

showfoil.pl - display an airfoil.dat file on the screen.

=head1 SYNOPSIS

	perl showfoil.pl airfoil-coords-file.dat

or

	showfoil.exe airfoil-coords-file.dat

run this program from the command line in the same folder as Xfoil.exe, as above.
Get perl from here if you need it: http://www.activestate.com/activeperl/downloads

     Options:
	(none)		run without options to be prompted for everything.
       -help	-?	brief help message
       -x		size of the display-area create (x) (actual window will be bigger to accomdate decorations/buttons etc).
       -y		size of the display-window to create (y)
       -s		spline factor (0=4-point scheme [NDG87, DD89](default), 0.5 is the scheme promoted in [Ros04], and 1 is the uniform cubic B-spline scheme [Sab02, LR80])
			factor 0 forces the spline to pass through your supplied coords exactly.  larger numbers will "bend" inside your coods, missing many of them to stay smoothe (resultant usage might require re-normalizing)
       -l		spline subdivision level (default 2, can use 1 though 5+ if necessary (larger 'l' on sparse data=smooth output, larger 'l' on big data makes surfaces "bumpy")
       -noline		don't "connect the dots" with straight green lines
       -man		full documentation

=head1 CREDITS

Adapted from http://www.perlmonks.org/?node_id=980572

=head1 DESCRIPTION

Uses Gtk2 windows to plot airfoil datapoints.

=head1 SUBROUTINES

=cut

######################################################################

use strict;
use warnings;		# same as -w switch above

use Glib qw/TRUE FALSE/;
use Gtk2 -init;

use Getopt::Long;	# Commandline argument parsing
use Pod::Usage;		# Inbuilt documentation helper

my %arg;		# We store copies of commandline arguments in here

# Parse options, and print help/manual if requested.
&GetOptions('help|?' => \$arg{'help'},			# breif instructions
	    'man' => \$arg{'man'},			# complete manual
	    'x=i'=>\$arg{'x'},				# screen X
	    'y=i'=>\$arg{'y'},				# screen Y
	    'l=i'=>\$arg{'l'},				# screen Y
	    's=s'=>\$arg{'s'},				# spline type
	    'noline'=>\$arg{'noline'},			# don't "connect the dots" with straight green lines
	   ) or &pod2usage(2); 
# no warnings;
	   &pod2usage(1) if ($arg{'help'});
	   &pod2usage(-exitstatus => 0, -verbose => 2) if $arg{'man'};
# use warnings;



my $xsize             = 640; $xsize=$arg{'x'} if($arg{'x'});
my $ysize             = 480; $ysize=$arg{'y'} if($arg{'y'});
my $subdivision_level = 2; $subdivision_level =$arg{'l'} if($arg{'l'});

my $winx=$xsize+30;	# 640 yeilds 648 (displayable X=610)
my $winy=$ysize+81;	# 480 yeilds 509 (displayable Y=399)


die "usage: $0 filename.dat" unless($ARGV[0]);

open(IN,'<',$ARGV[0]) or die "Cannot read input airfoil data file '$ARGV[0]': $!";

my @foil=<IN>; close(IN);

$foil[0]='' unless($foil[0]=~/^\s*[\d\-]/); # ignore 1st line if it doesn't begin with a number

my @points;

my ($xo,$yo,$xm,$ym)=($xsize/2,$ysize/2,$xsize/1.02,-$xsize/1.02);	# ym scaled by $xsize to retain aspect ratio

foreach (@foil) {
  next if (/^$/);
  $_=~s/^\s+//;
  my ($x,$y)=split(/\s+/);
  warn "file not normalized: found x=$x" if(($x<0)||($x>1));
  print "($x,$y)\n";

#warn "x=$x y=$y ym=$ym yo=$yo xm=$xm xo=$xo";
  $x=($x-0.5)*$xm+$xo;
  $y=$y*$ym+$yo;
#warn "xy=($x,$y)";

  push @points, $x;
  push @points, $y;
}
my $area;

my %allocated_colors;
my ( $x0, $y0, $x1, $y1, $width, ) = ( 0, 0, 0, 0 );

my $window = new Gtk2::Window("toplevel");
$window->signal_connect( "delete_event", sub { Gtk2->main_quit; } );
$window->set_border_width(10);
$window->set_size_request( $winx, $winy );
$window->set_position('center');

my $vbox = Gtk2::VBox->new( 0, 0 );
$window->add($vbox);
$vbox->set_border_width(2);

my $hbox = Gtk2::HBox->new( 0, 0 );
$vbox->pack_start( $hbox, 1, 1, 0 );
$hbox->set_size_request( 320, 240 );
$hbox->set_border_width(2);

my $hbox1 = Gtk2::HBox->new( 0, 0 );
$vbox->pack_start( $hbox1, 0, 0, 0 );
$hbox1->set_border_width(2);

my $button1 = Gtk2::Button->new('Generate');
$hbox1->pack_start( $button1, FALSE, FALSE, 2 );
$button1->signal_connect( clicked => sub { start_drawing($area) } );

my $button2 = Gtk2::Button->new('Exit');
$hbox1->pack_start( $button2, FALSE, FALSE, 2 );
$button2->signal_connect( clicked => sub { exit; } );

my $scwin = Gtk2::ScrolledWindow->new();
my $ha1   = $scwin->get_hadjustment;
$scwin->set_policy( 'never', 'never' );
#$scwin->set_policy( 'always', 'never' );

my $vp = Gtk2::Viewport->new( undef, undef );
$scwin->add($vp);
$hbox->pack_start( $scwin, 1, 1, 0 );

$area = new Gtk2::DrawingArea;
$area->size( $xsize, $ysize );
$vp->add($area);

$area->set_events(
	[
		qw/exposure-mask
		  leave-notify-mask
		  button-press-mask
		  pointer-motion-mask
		  pointer-motion-hint-mask/
	]
);

$area->signal_connect( button_press_event => \&button_press_event );



$window->show_all;

&draw_line( $scwin, [@points], 'green' );



Gtk2->main;

sub get_color {
	my ( $colormap, $name ) = @_;
	my $ret;

	if ( $ret = $allocated_colors{$name} ) {
		return $ret;
	}

	my $color = Gtk2::Gdk::Color->parse($name);
	$colormap->alloc_color( $color, TRUE, TRUE );

	$allocated_colors{$name} = $color;

	return $color;
}

sub draw_line {
	my ( $widget, $line, $color ) = @_;

	my $colormap = $widget->window->get_colormap;

	my $gc = $widget->{gc} || new Gtk2::Gdk::GC $widget->window;
	$gc->set_foreground( get_color( $colormap, $color ) );

	$widget->window->draw_lines( $gc, @$line );
	$gc->set_foreground( get_color( $colormap, 'black' ) );
	$widget->window->draw_points( $gc, @$line );
}

sub start_drawing {
	my $area = shift;

	&draw_line( $area, [@points], 'green' ) unless($arg{'noline'});

	my @xy; my $t=''; foreach (@points) {if($t eq '') {$t.=$_} else {push @xy,"$t,$_";$t=''}}
	my $lx=int(($ysize/200)+0.5); # length of the "+" marker lines
	foreach (@xy) {
	  my($x,$y)=split(/,/);
	  &draw_line( $area, [$x,$y-$lx,$x,$y+$lx], 'red' );
	  &draw_line( $area, [$x-$lx,$y,$x+$lx,$y], 'red' );
	}

#&draw_line( $area, [2,2,638,2,638,478,2,478,2,2], 'red' );
# &draw_line( $area, [0,0,$xsize-1,0,$xsize-1,$ysize-1,0,$ysize-1,0,0], 'red' );

&draw_line( $area, [$xsize/2-1,$ysize/2,$xsize/2+1,$ysize/2], 'red' );

&draw_line( $area, [0,$ysize/2,$xsize-1,$ysize/2], 'yellow' );
&draw_line( $area, [$xsize/2,0,$xsize/2,$ysize-1], 'yellow' );

	my $a;
	my $b;
	my $k = 0;
	while ( $k++ < $subdivision_level ) {
		my $j = 0;
		my @tmp;
		while ( $j < $#points ) {
			$a = $b = 0;
			if(defined $arg{'s'}) {$a = $b = $arg{'s'}}
			if ( $j == 0 ) {			# Anchor start of output line to the start point

				push( @tmp, $points[$j] );
				push( @tmp, $points[ $j + 1 ] );
print "a $j\t(" . $points[$j] . "," . $points[$j+1] . ")\n";
			}
			elsif ( $j + 3 <= $#points ) {		# 
				my ( $pt1, $pt2 );

				$pt1 =
				  ( $a * $points[ $j - 2 ] +
					  ( 8 - 2 * $a ) * $points[$j] +
					  $a * $points[ $j + 2 ] ) / 8;
				$pt2 =
				  ( $a * $points[ $j - 1 ] +
					  ( 8 - 2 * $a ) * $points[ $j + 1 ] +
					  $a * $points[ $j + 3 ] ) / 8;
				push( @tmp, int($pt1) );
				push( @tmp, int($pt2) );

print "b $j\t($pt1,$pt2)\n";

			}
			if ( $j + 5 <= $#points && $j > 0) {
				my ( $pt1, $pt2 );

				#push( @tmp, $points[ $j + 2 ] );
				#push( @tmp, $points[ $j + 3 ] );

				$pt1 =
				  ( ( $b - 1 ) * $points[$j -2] +
					  ( 9 - $b ) * $points[ $j ] +
					  ( 9 - $b ) * $points[ $j + 2 ] +
					  ( $b - 1 ) * $points[ $j + 4 ] ) / 16;
				$pt2 =
				  ( ( $b - 1 ) * $points[ $j -1 ] +
					  ( 9 - $b ) * $points[ $j + 1 ] +
					  ( 9 - $b ) * $points[ $j + 3 ] +
					  ( $b - 1 ) * $points[ $j + 5 ] ) / 16;
				push( @tmp, int($pt1) );
				push( @tmp, int($pt2) );
print "c $j\t($pt1,$pt2)\n";
			}
			$j += 2;
		}
		push( @tmp, $points[ $#points - 1 ] );
		push( @tmp, $points[$#points] );
print "d $j\t(" . $points[ $#points - 1 ] . ",$points[$#points])\n";
		@points = ();
		@points = @tmp;
		#print "@points\n";
	}

	&draw_line( $area, [@points], 'blue' );
#	print "@points";
}




=head1 AUTHOR

C. N. Drake, E<lt>christopher@pobox.comE<gt>

=head1 WARRANTY and DISCLAIMER

Important. There is no warranty (see License below).

USE AT OWN RISK!

=head1 COPYRIGHT AND LICENSE

This is distrubed under the GNU Public License:-
http://www.gnu.org/copyleft/gpl.html

Copyright (C) 2013 by C. N. Drake.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.3 or,
at your option, any later version of Perl 5 you may have available.

=cut
